//
//    **                                                **
//   ******************************************************
//  ***  THIS PROGRAM IS THE PROPERTY OF ALCYONE LIMITED ***
//   ******************************************************
//    **                                                **
//
//  tcuscan.c by Phil Skuse
//  Copyright (C) Alcyone Limited 2008.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//


#include <curses.h>
#include <stdlib.h>
#include <getopt.h>
#include <unistd.h>

void bar(int percent)
{
    int x,y;
    attron(COLOR_PAIR(3));
    for (x=0;x<percent/5;x++) printw(" ");
    attron(COLOR_PAIR(4));
    for (y=x;y<20;y++) printw(" ");
    attron(COLOR_PAIR(1));
}

void display_data()
{
    unsigned char data;
    unsigned char stick,fwd=0,man=0,pwr=0,crs=0,abs=0,brk=0,gear=0,mode=0;
    unsigned char sol1=0,sol2=0,sol3=0;
    int rc,DutyA,DutyB,DutyC;

    /*------------------------------------------*/
    /* Read FWD, Power, Cruise and ABS Switches */
    /*------------------------------------------*/

    if ((rc=ssm_query_tcu(0x0011,&data,1)) != 0)
    {
        printf("ssm_query_tcu() returned %d\n",rc);
        ssm_close();
        exit(7);
    }
    if (data & 0x80) fwd=1; else fwd=0;
    if (data & 0x40) pwr=1; else pwr=0;
    if (data & 0x20) crs=1; else crs=0;
    if (data & 0x10) abs=1; else abs=0;

    move(22,13);
    if (fwd == 1) attron(COLOR_PAIR(5)); else attron(COLOR_PAIR(1));
    printw(" FWD ");

    move(22,23);
    if (pwr == 1) attron(COLOR_PAIR(3)); else attron(COLOR_PAIR(1));
    printw(" PWR ");

    move(22,43);
    if (crs == 1) attron(COLOR_PAIR(3)); else attron(COLOR_PAIR(1));
    printw(" CRS ");

    move(22,63);
    if (abs == 1) attron(COLOR_PAIR(5)); else attron(COLOR_PAIR(1));
    printw(" ABS ");

    refresh();

    /*------------------------------*/
    /* Read Manual Switch and Stick */
    /*------------------------------*/

    if ((rc=ssm_query_tcu(0x0012,&data,1)) != 0)
    {
        printf("ssm_query_tcu() returned %d\n",rc);
        ssm_close();
        exit(7);
    }
    if (data & 0x40) man=1; else man=0;
    switch (data & 0x3F)
    {
        case 0x20: stick='1'; break;
        case 0x10: stick='2'; break;
        case 0x08: stick='3'; break;
        case 0x04: stick='D'; break;
        case 0x02: stick='R'; break;
        case 0x01: stick='N'; break;
        default  : stick=' '; break;
    }

    move(4,39);
    if (stick == 'R') attron(COLOR_PAIR(3)); else attron(COLOR_PAIR(1));
    printw(" R ");

    move(5,38);
    if (stick == 'N') attron(COLOR_PAIR(4)); else attron(COLOR_PAIR(1));
    printw(" N/P ");

    move(6,39);
    if (stick == 'D') attron(COLOR_PAIR(3)); else attron(COLOR_PAIR(1));
    printw(" D ");

    move(7,39);
    if (stick == '3') attron(COLOR_PAIR(3)); else attron(COLOR_PAIR(1));
    printw(" 3 ");

    move(8,39);
    if (stick == '2') attron(COLOR_PAIR(3)); else attron(COLOR_PAIR(1));
    printw(" 2 ");

    move(9,39);
    if (stick == '1') attron(COLOR_PAIR(3)); else attron(COLOR_PAIR(1));
    printw(" 1 ");

    move(22,33);
    if (man == 1) attron(COLOR_PAIR(4)); else attron(COLOR_PAIR(1));
    printw(" MAN ");

    refresh();

    /*-------------------*/
    /* Read Brake Switch */
    /*-------------------*/

    if ((rc=ssm_query_tcu(0x0013,&data,1)) != 0)
    {
        printf("ssm_query_tcu() returned %d\n",rc);
        ssm_close();
        exit(7);
    }
    if (data & 0x04) brk=1; else brk=0;

    move(22,53);
    if (brk == 1) attron(COLOR_PAIR(4)); else attron(COLOR_PAIR(1));
    printw(" BRK ");

    refresh();

    /*-------------------*/
    /* Read Current Gear */
    /*-------------------*/

    if ((rc=ssm_query_tcu(0x004E,&data,1)) != 0)
    {
        printf("ssm_query_tcu() returned %d\n",rc);
        ssm_close();
        exit(7);
    }
    if (data < 4) gear=data+1; else gear=0;

    move(20,12);
    if (gear == 1) attron(COLOR_PAIR(6)); else attron(COLOR_PAIR(1));
    printw(" 1ST ");

    move(20,22);
    if (gear == 2) attron(COLOR_PAIR(6)); else attron(COLOR_PAIR(1));
    printw(" 2ND ");

    move(20,32);
    if (gear == 3) attron(COLOR_PAIR(6)); else attron(COLOR_PAIR(1));
    printw(" 3RD ");

    move(20,42);
    if (gear == 4) attron(COLOR_PAIR(6)); else attron(COLOR_PAIR(1));
    printw(" 4TH ");

    refresh();

    /*--------------------*/
    /* Read Shift Pattern */
    /*--------------------*/

    if ((rc=ssm_query_tcu(0x005A,&data,1)) != 0)
    {
        printf("ssm_query_tcu() returned %d\n",rc);
        ssm_close();
        exit(7);
    }
    if (data >0) mode=1; else mode=0;

    move(20,52);
    if (mode == 0) attron(COLOR_PAIR(4)); else attron(COLOR_PAIR(1));
    printw(" NORMAL ");

    move(20,62);
    if (mode == 1) attron(COLOR_PAIR(3)); else attron(COLOR_PAIR(1));
    printw(" POWER ");

    refresh();

    /*----------------------*/
    /* Read SolA Duty Cycle */
    /*----------------------*/

    if ((rc=ssm_query_tcu(0x00B3,&data,1)) != 0)
    {
        printf("ssm_query_tcu() returned %d\n",rc);
        ssm_close();
        exit(7);
    }
    DutyA=data*5;

    move(4,8);
    attron(COLOR_PAIR(1));
    printw("Duty Solenoid A");
    move(5,5);
    bar(DutyA/10);
    printw(" %d.%d%% ",DutyA/10,DutyA%10);

    refresh();

    /*----------------------*/
    /* Read SolB Duty Cycle */
    /*----------------------*/

    if ((rc=ssm_query_tcu(0x00B4,&data,1)) != 0)
    {
        printf("ssm_query_tcu() returned %d\n",rc);
        ssm_close();
        exit(7);
    }
    DutyB=data*5;

    move(7,8);
    printw("Duty Solenoid B");
    move(8,5);
    bar(DutyB/10);
    printw(" %d.%d%% ",DutyB/10,DutyB%10);

    refresh();

    /*----------------------*/
    /* Read SolC Duty Cycle */
    /*----------------------*/

    if ((rc=ssm_query_tcu(0x00B5,&data,1)) != 0)
    {
        printf("ssm_query_tcu() returned %d\n",rc);
        ssm_close();
        exit(7);
    }
    DutyC=data*5;

    move(10,8);
    printw("Duty Solenoid C");
    move(11,5);
    bar(DutyC/10);
    printw(" %d.%d%% ",DutyC/10,DutyC%10);

    refresh();

    /*----------------------*/
    /* Read Shift Solenoids */
    /*----------------------*/

    if ((rc=ssm_query_tcu(0x0060,&data,1)) != 0)
    {
        printf("ssm_query_tcu() returned %d\n",rc);
        ssm_close();
        exit(7);
    }
    if (data & 0x80) sol3=1; else sol3=0;
    if (data & 0x20) sol2=1; else sol2=0;
    if (data & 0x10) sol1=1; else sol1=0;

    move(13,8);
    attron(COLOR_PAIR(1));
    printw("Shift Solenoids");
    move(15,9);
    if (sol1 == 1) attron(COLOR_PAIR(4)); else attron(COLOR_PAIR(1));
    printw(" 1 ");
    move(15,14);
    if (sol2 == 1) attron(COLOR_PAIR(4)); else attron(COLOR_PAIR(1));
    printw(" 2 ");
    move(15,19);
    if (sol3 == 1) attron(COLOR_PAIR(4)); else attron(COLOR_PAIR(1));
    printw(" 3 ");

    refresh();
}

int main(int argc, char *argv[])
{
    char rc, opt;
    char *comport="/dev/ttyS0";

    while ((opt = getopt (argc, argv, "d:")) != -1 )
    {
        switch(opt)
        {
            case 'd': comport=optarg;
                      break;
            default : printf("Usage: tcuscan [-d serial device]\n");
                      exit(0);
        }
    }

    if ((rc=ssm_open(comport)) != 0)
    {
        printf("ssm_open() returned %d\n",rc);
        perror("Error Details");
        exit(1);
    }

    initscr();
    curs_set(0);
    start_color();
    init_pair(1,COLOR_WHITE,COLOR_BLACK);
    init_pair(2,COLOR_GREEN,COLOR_BLACK);
    init_pair(3,COLOR_BLACK,COLOR_GREEN);
    init_pair(4,COLOR_BLACK,COLOR_YELLOW);
    init_pair(5,COLOR_BLACK,COLOR_RED);
    init_pair(6,COLOR_BLACK,COLOR_CYAN);
    move(0,24);
    printw("Subaru 4EAT Transmission Utility");
    move(1,24);
    printw("================================");
    while(1)
    {
        display_data();
        refresh();
    }
    getch();
    endwin();
    ssm_close();
}
